<?php

namespace Bitrix\Modules\Sale\Handler\Paysystem\Paykeeper;

use Bitrix\Sale\BusinessValue;
use Bitrix\Sale\Order;
use Bitrix\Sale\PaySystem\Service;
use Bitrix\Main\Web\HttpClient;
use Bitrix\Main\Event;
use Bitrix\Main\EventResult;
use Bitrix\Main\EventManager;
use Bitrix\Main\Loader;
use Bitrix\Main\Context;

use Bitrix\Sale\Internals\PaymentTable;
use Bitrix\Sale\Internals\PayableItemTable;

// Registering the event handler for the 'OnSaleStatusOrder' event in the 'sale' module.
EventManager::getInstance()->addEventHandler(
    'sale',
    'OnSaleStatusOrder',
    [__NAMESPACE__ . '\PaykeeperEventHandler','OnSaleStatusOrderChange']
);

/**
 * Uncommented test method for the custom event handler.
 *
 * This code snippet demonstrates how to register a custom event handler for
 * the 'onPrintCheckPaykeeper' event in the 'paykeeper' module. The handler method
 * (PaykeeperEventHandler::checkEventTest) will be executed when this event is triggered.
 */
/*EventManager::getInstance()->addEventHandler(
    'paykeeper',
    'onPrintCheckPaykeeper',
    [__NAMESPACE__ . '\PaykeeperEventHandler','checkEventTest']
);*/

/**
 * Class PaykeeperEventHandler
 * Handles events and actions related to Paykeeper integration.
 */
class PaykeeperEventHandler {

    /**
     * Event handler for the OnSaleStatusOrderChange event.
     *
     * This method is triggered when the order status changes. It handles the final receipt generation
     * and interaction with the Paykeeper API if certain conditions are met.
     *
     * @param int $orderId The ID of the order whose status has changed.
     * @return EventResult Returns an EventResult object, indicating the success of the operation.
     */
    static function OnSaleStatusOrderChange($orderId)
    {
        $order = Order::load($orderId);
        if (!$order->isPaid()) {
            return self::getSuccessEventResult();
        }

        $paySystemId = $order->getField('PAY_SYSTEM_ID');
        $personTypeId = $order->getField('PERSON_TYPE_ID');

        $paykeeper_print_final_receipt = BusinessValue::get(
            'PAYKEEPER_PRINT_FINAL_RECEIPT',
            Service::PAY_SYSTEM_PREFIX . $paySystemId,
            $personTypeId
        );

        if ($paykeeper_print_final_receipt != 'Y') {
            return self::getSuccessEventResult();
        }

        $paykeeper_url = BusinessValue::get(
            'PAYKEEPER_FORM_URL',
            Service::PAY_SYSTEM_PREFIX . $paySystemId,
            $personTypeId
        );

        if (!$paykeeper_url) {
            return self::getSuccessEventResult();
        }

        if ($pos = strrpos($paykeeper_url, '/create') ?: strrpos($paykeeper_url, '/order')) {
            $paykeeper_url = substr($paykeeper_url, 0, $pos);
        }

        $paykeeper_print_after_status = BusinessValue::get(
            'PAYKEEPER_PRINT_AFTER_STATUS',
            Service::PAY_SYSTEM_PREFIX . $paySystemId,
            $personTypeId
        );

        $paykeeper_lk_login = BusinessValue::get(
            'PAYKEEPER_LK_LOGIN',
            Service::PAY_SYSTEM_PREFIX . $paySystemId,
            $personTypeId
        );

        $paykeeper_lk_password = BusinessValue::get(
            'PAYKEEPER_LK_PASSWORD',
            Service::PAY_SYSTEM_PREFIX . $paySystemId,
            $personTypeId
        );

        if (!$paykeeper_lk_login ||
            !$paykeeper_lk_password ||
            $paykeeper_print_after_status != $order->getField('STATUS_ID'))
        {
            return self::getSuccessEventResult();
        }

        // Получаем ID платежа РК
        $paymentCollection = $order->getPaymentCollection();
        $paymentId = 0;
        foreach ($paymentCollection as $payment) {
            if ($payment->getField('PAY_SYSTEM_ID') != $paySystemId) {
                continue;
            }
            $paymentId = preg_replace('/\D/', '', $payment->getField('PS_STATUS_MESSAGE'));
        }

        if (!$paymentId) {
            return self::getSuccessEventResult();
        }

        /* Авторизация в Paykeeper */
        $paykeeper_nttp_client = new HttpClient();
        $basic_auth = base64_encode("$paykeeper_lk_login:$paykeeper_lk_password");
        $paykeeper_nttp_client->setHeader('Content-Type','application/x-www-form-urlencoded');
        $paykeeper_nttp_client->setHeader('Authorization','Basic '.$basic_auth);
        $result_paykeeper_get = $paykeeper_nttp_client->get("$paykeeper_url/info/settings/token/");
        $response = json_decode($result_paykeeper_get, true);

        /* В ответе должен быть token, иначе ошибка */
        if (isset($response['token'])) {
            $token = $response['token'];

            $response = $paykeeper_nttp_client->post(
                "$paykeeper_url/change/payment/post-sale-receipt/",
                array_merge([ 'id' => $paymentId ], array('token' => $token))
            );
            $response = json_decode($response, true);

            // Создаем новую группу свойств и свойство для чека окончательного расчета, если их нет
            $propertyCode = 'PAYKEEPER_RECEIPT_ID';
            $propertyName = 'Номер запроса на генерацию чека в PK';
            self::createOrderPropertyForAllPersonTypes($propertyName, $propertyCode);

            $receipt_id = 0;
            /* В ответе должен быть receipt_id */
            if (isset($response['receipt_id'])) {
                $receipt_id = $response['receipt_id'];
                $infoLog = 'Заказ '.$orderId.'. ID чека ' . $receipt_id;

                // Записываем значение номера запроса печати чека
                /** @var \Bitrix\Sale\PropertyValueCollection $propertyCollection */
                $propertyCollection = $order->getPropertyCollection();
                $property = $propertyCollection->getItemByOrderPropertyCode($propertyCode);

                if($property && $property->getValue() != $receipt_id) {
                    $property->setField('VALUE', $receipt_id);
                    $order->save();
                }
            } elseif (isset($response['error'])) {
                $infoLog = 'Заказ '.$orderId.'. Ошибка: '.$response['error'];
                self::triggerPaykeeperPrintCheckError($infoLog, $orderId);
            } elseif (isset($response['msg'])) {
                $infoLog = 'Заказ '.$orderId.'. Ошибка: '.$response['msg'];
                self::triggerPaykeeperPrintCheckError($infoLog, $orderId);
            } else {
                $infoLog = 'Заказ '.$orderId.'. Неизвестная ошибка!';
                self::triggerPaykeeperPrintCheckError($infoLog, $orderId);
            }
        } else {
            $infoLog = 'Заказ '.$orderId.'. Ошибка авторизации!';
            self::triggerPaykeeperPrintCheckError($infoLog, $orderId);
        }

        // Добавляем запись в журнал событий об API-запросе
        \CEventLog::Add(array(
            'SEVERITY' => 'INFO',
            'AUDIT_TYPE_ID' => 'Печать чека окончательного расчета PayKeeper',
            'MODULE_ID' => 'sale',
            //'ITEM_ID' => __METHOD__,
            'ITEM_ID' => 'API запрос /change/payment/post-sale-receipt/',
            'DESCRIPTION' => $infoLog,
        ));

        return self::getSuccessEventResult();
    }

    /**
     * Creates an order property for all person types.
     *
     * This method checks if a property with a specific code exists for each person type in the system.
     * If not, it creates the property and assigns it to a property group.
     *
     * @param string $propertyName The name of the property to be created.
     * @param string $propertyCode The unique code of the property.
     * @return void
     */
    static function createOrderPropertyForAllPersonTypes($propertyName, $propertyCode)
    {
        // Получаем все типы плательщиков
        $personTypes = [];
        $dbPersonTypes = \CSalePersonType::GetList( ['SORT' => 'ASC'], [] );
        while ($personType = $dbPersonTypes->Fetch()) {
            $personTypes[] = $personType['ID'];
        }

        foreach ($personTypes as $personTypeId) {
            // Проверяем, существует ли уже свойство с таким кодом для данного типа плательщика
            $dbProps = \CSaleOrderProps::GetList(
                [],
                [ 'CODE' => $propertyCode, 'PERSON_TYPE_ID' => $personTypeId ]
            );

            if ($dbProps->Fetch())
                continue;

            // Создаем новую группу свойства
            $groupExists = false;
            $propsGroupId = null;
            $nameGroup = 'Генерация чека окончательного расчета';
            $groupCheck = \CSaleOrderPropsGroup::GetList( [],
                [ 'NAME' => $nameGroup, 'PERSON_TYPE_ID' => $personTypeId ]
            );

            if ($group = $groupCheck->Fetch()) {
                $groupExists = true;
                $propsGroupId = $group['ID'];
            }

            if (!$groupExists) {
                $groupFields = [
                    'NAME' => $nameGroup,
                    'PERSON_TYPE_ID' => $personTypeId,
                    'SORT' => 500
                ];
                $propsGroupId = \CSaleOrderPropsGroup::Add($groupFields);
            }

            // Создаем новое свойство
            $propertyFields = array(
                'NAME' => $propertyName,
                'TYPE' => 'STRING', // Тип свойства: STRING, NUMBER, FILE, etc.
                'REQUIRED' => 'N', // Обязательное: Y или N
                'DEFAULT_VALUE' => '',
                'SORT' => 500,
                'USER_PROPS' => 'Y', // Включить в свойства пользователя: Y или N
                'IS_LOCATION' => 'N',
                'IS_LOCATION4TAX' => 'N',
                'IS_EMAIL' => 'N',
                'IS_PROFILE_NAME' => 'N',
                'IS_PAYER' => 'N',
                'IS_FILTERED' => 'N',
                'CODE' => $propertyCode,
                'ACTIVE' => 'Y',
                'UTIL' => 'Y',
                'INPUT_FIELD_LOCATION' => '',
                'PROPS_GROUP_ID' => $propsGroupId, // ID группы свойств
                'SIZE1' => 0,
                'SIZE2' => 0,
                'DESCRIPTION' => '',
                'PERSON_TYPE_ID' => $personTypeId // Тип плательщика
            );
            \CSaleOrderProps::Add($propertyFields);
        }
    }

    /**
     * Triggers an event in case of an error in the Paykeeper API request.
     *
     * This method logs the error message and triggers the custom event 'onPrintCheckPaykeeper'.
     * Additional error handling can be performed in the event handler.
     *
     * @param string $errMessage The error message received from the Paykeeper API.
     * @param string $orderId The ID of the order that encountered the error.
     * @return void
     */
    static function triggerPaykeeperPrintCheckError($errMessage, $orderId) {
        $event = new Event('paykeeper', 'onPrintCheckPaykeeper', [ $errMessage, $orderId ]);
        $event->send();

        foreach ($event->getResults() as $eventResult) {
            if ($eventResult->getType() == EventResult::ERROR) {
                // здесь обработка ошибки
            }
        }
    }

    // Uncommented test method for the custom event
    /**
     * Test method for the 'onPrintCheckPaykeeper' event.
     *
     * This method demonstrates how the custom event can be tested and error handling
     * can be performed within the event.
     *
     * @param \Bitrix\Main\Event $event The event object containing parameters and results.
     * @return EventResult Returns an EventResult object, indicating success or error.
     */
    static function checkEventTest($event)
    {
        $resultType = $event->getEventType() ?? EventResult::SUCCESS;
        try {
            [ $errMessage, $orderId ] = $event->getParameters();

            // здесь обработчик
        } catch (\Exception $e){
            $resultType = EventResult::ERROR;
        }

        return new EventResult($resultType);
    }

    /**
     * @return EventResult
     */
    private static function getSuccessEventResult() {
        return new EventResult(EventResult::SUCCESS);
    }
}
